import sys

EPSILON: float = 0.0000001

MIN_FLT: float = sys.float_info.min
MAX_FLT: float = sys.float_info.max

MIN_ROOF_EDGE_DISTANCE = 0.3
ROOF_MIN_ANGLE = 0.0017
ROOF_MAX_SCAN = 20

LIGHT_SPEED = 299792458
